@@ -1,11 +1,9 @@
-# -*- coding: utf8 -*-
-from __future__ import unicode_literals
-
 import json
 
 from django import template
 from django.conf import settings
-from django.utils import six
+from django.core.serializers.json import DjangoJSONEncoder
+from django.utils.encoding import force_text
 
 from leaflet import (app_settings, SPATIAL_EXTENT, SRID, PLUGINS, PLUGINS_DEFAULT,
                      PLUGIN_ALL, PLUGIN_FORMS)
@@ -38,59 +36,79 @@ def leaflet_js(plugins=None):
     """
     plugin_names = _get_plugin_names(plugins)
     with_forms = PLUGIN_FORMS in plugin_names or PLUGIN_ALL in plugin_names
+    FORCE_IMAGE_PATH = app_settings.get('FORCE_IMAGE_PATH')
+    template_options = settings.TEMPLATES[0].get('OPTIONS', None)
+
+    if template_options and 'debug' in template_options:
+        debug = template_options['debug']
+    else:
+        debug = False
+
     return {
-        "DEBUG": settings.TEMPLATE_DEBUG,
+        "DEBUG": debug,
         "SRID": str(SRID) if SRID else None,
         "PLUGINS_JS": _get_all_resources_for_plugins(plugin_names, 'js'),
-        "with_forms": with_forms
+        "with_forms": with_forms,
+        "FORCE_IMAGE_PATH": FORCE_IMAGE_PATH
     }
 
 
 @register.inclusion_tag('leaflet/_leaflet_map.html')
-def leaflet_map(name, callback=None, fitextent=True, creatediv=True, loadevent='load'):
+def leaflet_map(name, callback=None, fitextent=True, creatediv=True,
+                loadevent=app_settings.get('LOADEVENT'),
+                settings_overrides={}):
     """
 
     :param name:
     :param callback:
     :param fitextent:
     :param creatediv:
+    :param loadevent:
+    :param settings_overrides:
     :return:
     """
+
+    if settings_overrides == '':
+        settings_overrides = {}
+
+    instance_app_settings = app_settings.copy()  # Allow not overidding global app_settings
+    instance_app_settings.update(**settings_overrides)
+
     extent = None
-    if SPATIAL_EXTENT is not None:
+    if instance_app_settings['SPATIAL_EXTENT'] is not None:
         # Leaflet uses [lat, lng]
-        xmin, ymin, xmax, ymax = SPATIAL_EXTENT
-        extent = (ymin, xmin, ymax, xmax)
+        xmin, ymin, xmax, ymax = instance_app_settings['SPATIAL_EXTENT']
+        bbox = (ymin, xmin, ymax, xmax)
+        extent = [bbox[:2], bbox[2:4]]
 
     djoptions = dict(
         srid=SRID,
-        extent=[extent[:2], extent[2:4]],
+        extent=extent,
         fitextent=fitextent,
-        center=app_settings['DEFAULT_CENTER'],
-        zoom=app_settings['DEFAULT_ZOOM'],
-        layers=[(six.text_type(label), url, attrs) for (label, url, attrs) in app_settings.get('TILES')],
-        overlays=[(six.text_type(label), url, attrs) for (label, url, attrs) in app_settings.get('OVERLAYS')],
-        attributionprefix=app_settings.get('ATTRIBUTION_PREFIX'),
-        scale=app_settings.get('SCALE'),
-        minimap=app_settings.get('MINIMAP'),
-        resetview=app_settings.get('RESET_VIEW'),
-        tilesextent=list(app_settings.get('TILES_EXTENT', []))
+        center=instance_app_settings['DEFAULT_CENTER'],
+        zoom=instance_app_settings['DEFAULT_ZOOM'],
+        precision=instance_app_settings['DEFAULT_PRECISION'],
+        minzoom=instance_app_settings['MIN_ZOOM'],
+        maxzoom=instance_app_settings['MAX_ZOOM'],
+        layers=[(force_text(label), url, attrs) for (label, url, attrs) in instance_app_settings.get('TILES')],
+        overlays=[(force_text(label), url, attrs) for (label, url, attrs) in instance_app_settings.get('OVERLAYS')],
+        attributionprefix=force_text(instance_app_settings.get('ATTRIBUTION_PREFIX'), strings_only=True),
+        scale=instance_app_settings.get('SCALE'),
+        minimap=instance_app_settings.get('MINIMAP'),
+        resetview=instance_app_settings.get('RESET_VIEW'),
+        tilesextent=list(instance_app_settings.get('TILES_EXTENT', []))
     )
-    if app_settings['MIN_ZOOM'] is not None:
-        djoptions['minzoom'] = app_settings['MIN_ZOOM']
-    if app_settings['MAX_ZOOM'] is not None:
-        djoptions['maxzoom'] = app_settings['MAX_ZOOM']
 
     return {
         # templatetag options
         'name': name,
-        'loadevents': json.dumps(loadevent.split()),
+        'loadevents': json.dumps(loadevent.split(), cls=DjangoJSONEncoder),
         'creatediv': creatediv,
         'callback': callback,
         # initialization options
-        'djoptions': json.dumps(djoptions),
+        'djoptions': json.dumps(djoptions, cls=DjangoJSONEncoder),
         # settings
-        'NO_GLOBALS': app_settings.get('NO_GLOBALS'),
+        'NO_GLOBALS': instance_app_settings.get('NO_GLOBALS'),
     }
 
 
@@ -103,7 +121,7 @@ def leaflet_json_config():
         settings_as_json['SPATIAL_EXTENT'] = {'xmin': xmin, 'ymin': ymin,
                                               'xmax': xmax, 'ymax': ymax}
 
-    return json.dumps(settings_as_json)
+    return json.dumps(settings_as_json, cls=DjangoJSONEncoder)
 
 
 def _get_plugin_names(plugin_names_from_tag_parameter):
@@ -113,7 +131,7 @@ def _get_plugin_names(plugin_names_from_tag_parameter):
     :param pluging_names_parameter:
     :return:
     """
-    if isinstance(plugin_names_from_tag_parameter, (six.binary_type, six.text_type)):
+    if isinstance(plugin_names_from_tag_parameter, str):
         names = plugin_names_from_tag_parameter.split(',')
         return [n.strip() for n in names]
     else:
